/**
 * Utility function that uses the type system to check if a switch statement is exhaustive.
 * If the switch statement is not exhaustive, there will be a type error caught in typescript
 *
 * See https://stackoverflow.com/questions/39419170/how-do-i-check-that-a-switch-block-is-exhaustive-in-typescript for more details.
 */
export declare function assertUnreachable(_: never): never;
/**
 * A type-guard function for checking if a value is an object
 */
export declare function isObject(value: unknown): value is Record<string | number | symbol, unknown>;
export declare function isString(value: unknown): value is string;
export declare function isNumber(value: unknown): value is number;
export declare function isNonEmptyArray(value: unknown): value is unknown[];
//# sourceMappingURL=typeUtils.d.ts.map