"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.VERSION = void 0;
// this is an auto-generated file. see js/scripts/version-update.js
exports.VERSION = "2.0.0";
//# sourceMappingURL=version.js.map